@@ -32,12 +32,13 @@ if TYPE_CHECKING:
     from numpy.random import Generator
     from numpy.typing import NDArray
 
-    from ropt.config.enopt import EnOptConfig
-    from ropt.evaluator import Evaluator
+    from ropt.config import EnOptConfig
+    from ropt.evaluator import EvaluatorCallback
     from ropt.plugins import PluginManager
     from ropt.plugins.function_estimator.base import FunctionEstimator
     from ropt.plugins.realization_filter.base import RealizationFilter
     from ropt.plugins.sampler.base import Sampler
+    from ropt.transforms import OptModelTransforms
 
 
 class EnsembleEvaluator:
@@ -45,20 +46,21 @@ class EnsembleEvaluator:
 
     The `EnsembleEvaluator` class is responsible for calculating functions and
     gradients from an ensemble of functions. It leverages the settings defined
-    in an [`EnOptConfig`][ropt.config.enopt.EnOptConfig] configuration object to
-    guide the calculations.
+    in an [`EnOptConfig`][ropt.config.EnOptConfig] configuration object to guide
+    the calculations.
 
-    The core functionality relies on an [`Evaluator`][ropt.evaluator.Evaluator]
-    callable, which is used to evaluate the individual functions within the
-    ensemble. The evaluator provides the raw function values, which are then
-    processed by the `EnsembleEvaluator` to produce the final function and
-    gradient estimates.
+    The core functionality relies on an evaluator callable, (usually provided by
+    an [`Evaluator`][ropt.plugins.plan.base.Evaluator] object), which is used to
+    evaluate the individual functions within the ensemble. The evaluator
+    provides the raw function values, which are then processed by the
+    `EnsembleEvaluator` to produce the final function and gradient estimates.
     """
 
     def __init__(
         self,
         config: EnOptConfig,
-        evaluator: Evaluator,
+        transforms: OptModelTransforms | None,
+        evaluator: EvaluatorCallback,
         plugin_manager: PluginManager,
     ) -> None:
         """Initialize the EnsembleEvaluator.
@@ -68,21 +70,25 @@ class EnsembleEvaluator:
 
         The `config` object contains all the settings required for the ensemble
         evaluation, such as the number of realizations, the function estimators,
-        and the gradient settings. The `evaluator` callable should conform to
-        the [`Evaluator`][ropt.evaluator.Evaluator] protocol. The
+        and the gradient settings. The `transforms` object defines the domain
+        transforms that should be applied to variables, objectives and
+        constraints. The `evaluator` callable is usually provide by a
+        [`Evaluator`][ropt.plugins.plan.base.Evaluator] object. The
         `plugin_manager` is used to load the realization filters, function
         estimators, and samplers.
 
         Args:
             config:         The configuration object.
+            transforms:     The domain transforms to apply.
             evaluator:      The callable for evaluating individual functions.
             plugin_manager: A plugin manager to load required plugins.
         """
         self._config = config
+        self._transforms = transforms
         self._evaluator = evaluator
         self._realization_filters = self._init_realization_filters(plugin_manager)
         self._function_estimators = self._init_function_estimators(plugin_manager)
-        rng = default_rng(config.gradient.seed)
+        rng = default_rng(config.variables.seed)
         self._samplers = self._init_samplers(rng, plugin_manager)
         self._cache_for_gradient: FunctionResults | None = None
 
@@ -145,15 +151,17 @@ class EnsembleEvaluator:
     ) -> tuple[FunctionResults, ...]:
         if variables.ndim == 1:
             variables = variables[np.newaxis, :]
-        active_objectives, active_constraints = _get_active_realizations(self._config)
+        active_realizations = _get_active_realizations(self._config)
         function_results = tuple(
-            self._calculate_one_set_of_functions(f_eval_results, variables[idx, :])
+            self._calculate_one_set_of_functions(
+                f_eval_results, variables[idx, :], active_realizations
+            )
             for idx, f_eval_results in _get_function_results(
                 self._config,
+                self._transforms,
                 self._evaluator,
                 variables,
-                active_objectives,
-                active_constraints,
+                active_realizations,
             )
         )
 
@@ -165,7 +173,10 @@ class EnsembleEvaluator:
         return function_results
 
     def _calculate_one_set_of_functions(
-        self, f_eval_results: _FunctionEvaluatorResults, variables: NDArray[np.float64]
+        self,
+        f_eval_results: _FunctionEvaluatorResults,
+        variables: NDArray[np.float64],
+        active_realizations: NDArray[np.bool_],
     ) -> FunctionResults:
         objective_weights, constraint_weights = (
             self._calculate_filtered_realization_weights(f_eval_results)
@@ -203,8 +214,10 @@ class EnsembleEvaluator:
         return FunctionResults(
             batch_id=f_eval_results.batch_id,
             metadata={},
+            names=self._config.names,
             evaluations=evaluations,
             realizations=Realizations(
+                active_realizations=active_realizations,
                 failed_realizations=failed_realizations,
                 objective_weights=objective_weights,
                 constraint_weights=constraint_weights,
@@ -212,6 +225,7 @@ class EnsembleEvaluator:
             functions=functions,
             constraint_info=ConstraintInfo.create(
                 self._config,
+                self._transforms,
                 evaluations.variables,
                 functions.constraints if functions is not None else None,
             ),
@@ -236,17 +250,17 @@ class EnsembleEvaluator:
         objective_weights = self._cache_for_gradient.realizations.objective_weights
         constraint_weights = self._cache_for_gradient.realizations.constraint_weights
 
-        active_objectives, active_constraints = _get_active_realizations(
+        active_realizations = _get_active_realizations(
             self._config,
             objective_weights=objective_weights,
             constraint_weights=constraint_weights,
         )
         g_eval_results = _get_gradient_results(
             self._config,
+            self._transforms,
             self._evaluator,
             perturbed_variables,
-            active_objectives,
-            active_constraints,
+            active_realizations,
         )
 
         assert self._config.gradient.perturbation_min_success is not None
@@ -280,6 +294,7 @@ class EnsembleEvaluator:
             GradientResults(
                 batch_id=g_eval_results.batch_id,
                 metadata={},
+                names=self._config.names,
                 evaluations=GradientEvaluations.create(
                     variables=variables,
                     perturbed_variables=perturbed_variables,
@@ -288,6 +303,7 @@ class EnsembleEvaluator:
                     evaluation_info=g_eval_results.evaluation_info,
                 ),
                 realizations=Realizations(
+                    active_realizations=active_realizations,
                     failed_realizations=failed_realizations,
                     objective_weights=objective_weights,
                     constraint_weights=constraint_weights,
@@ -304,14 +320,14 @@ class EnsembleEvaluator:
         perturbed_variables = _perturb_variables(
             self._config, variables, self._samplers
         )
-        active_objectives, active_constraints = _get_active_realizations(self._config)
+        active_realizations = _get_active_realizations(self._config)
         f_eval_results, g_eval_results = _get_function_and_gradient_results(
             self._config,
+            self._transforms,
             self._evaluator,
             variables,
             perturbed_variables,
-            active_objectives,
-            active_constraints,
+            active_realizations,
         )
 
         evaluations = FunctionEvaluations.create(
@@ -349,8 +365,10 @@ class EnsembleEvaluator:
         function_results = FunctionResults(
             batch_id=f_eval_results.batch_id,
             metadata={},
+            names=self._config.names,
             evaluations=evaluations,
             realizations=Realizations(
+                active_realizations=active_realizations,
                 failed_realizations=failed_realizations,
                 objective_weights=objective_weights,
                 constraint_weights=constraint_weights,
@@ -358,6 +376,7 @@ class EnsembleEvaluator:
             functions=functions,
             constraint_info=ConstraintInfo.create(
                 self._config,
+                self._transforms,
                 evaluations.variables,
                 functions.constraints if functions is not None else None,
             ),
@@ -392,6 +411,7 @@ class EnsembleEvaluator:
         gradient_results = GradientResults(
             batch_id=g_eval_results.batch_id,
             metadata={},
+            names=self._config.names,
             evaluations=GradientEvaluations.create(
                 variables=variables,
                 perturbed_variables=perturbed_variables,
@@ -400,6 +420,7 @@ class EnsembleEvaluator:
                 evaluation_info=g_eval_results.evaluation_info,
             ),
             realizations=Realizations(
+                active_realizations=active_realizations,
                 failed_realizations=failed_realizations,
                 objective_weights=objective_weights,
                 constraint_weights=constraint_weights,
@@ -552,7 +573,7 @@ class EnsembleEvaluator:
     ) -> NDArray[np.float64]:
         if mask is None:
             return gradients
-        shape = gradients.shape[:-1] + (mask.size,)
+        shape = (*gradients.shape[:-1], mask.size)
         result = np.zeros(shape, dtype=np.float64)
         result[..., mask] = gradients
         return result
@@ -635,8 +656,8 @@ class EnsembleEvaluator:
     ) -> list[Sampler]:
         samplers: list[Sampler] = []
         for idx, sampler_config in enumerate(self._config.samplers):
-            variable_indices = _get_mask(
-                idx, self._config.gradient.samplers, self._config.variables.mask
+            variable_indices = np.asarray(
+                self._config.variables.mask & (self._config.variables.samplers == idx)
             )
             if variable_indices is None or variable_indices.size:
                 plugin = plugin_manager.get_plugin(
@@ -644,13 +665,3 @@ class EnsembleEvaluator:
                 )
                 samplers.append(plugin.create(self._config, idx, variable_indices, rng))
         return samplers
-
-
-def _get_mask(
-    idx: int, gradient_indices: NDArray[np.intc] | None, mask: NDArray[np.bool_] | None
-) -> NDArray[np.bool_] | None:
-    if gradient_indices is None:
-        return mask
-    if mask is None:
-        return np.asarray(gradient_indices == idx)
-    return np.asarray(mask & (gradient_indices == idx))
