@@ -1,9 +1,9 @@
 """Function evaluation protocols and classes."""
 
-from ._evaluator import Evaluator, EvaluatorContext, EvaluatorResult
+from ._evaluator import EvaluatorCallback, EvaluatorContext, EvaluatorResult
 
 __all__ = [
-    "Evaluator",
+    "EvaluatorCallback",
     "EvaluatorContext",
     "EvaluatorResult",
 ]
