@@ -9,8 +9,9 @@ from typing import TYPE_CHECKING, Protocol
 
 import numpy as np
 
-from ropt.enums import OptimizerExitCode
-from ropt.exceptions import OptimizationAborted
+from ropt.enums import ExitCode
+from ropt.exceptions import StepAborted
+from ropt.optimization import OptimizerCallbackResult
 from ropt.results import FunctionResults, GradientResults
 
 if TYPE_CHECKING:
@@ -18,11 +19,12 @@ if TYPE_CHECKING:
 
     from numpy.typing import NDArray
 
-    from ropt.config.enopt import EnOptConfig
+    from ropt.config import EnOptConfig
     from ropt.ensemble_evaluator import EnsembleEvaluator
     from ropt.plugins import PluginManager
     from ropt.plugins.optimizer.base import Optimizer
     from ropt.results import Functions, Gradients, Results
+    from ropt.transforms._transforms import OptModelTransforms
 
 
 class SignalEvaluationCallback(Protocol):
@@ -32,10 +34,7 @@ class SignalEvaluationCallback(Protocol):
     custom handling or tracking of evaluation events.
     """
 
-    def __call__(
-        self,
-        results: tuple[Results, ...] | None = None,
-    ) -> None:
+    def __call__(self, results: tuple[Results, ...] | None = None, /) -> None:
         """Callback protocol for signaling the start and end of evaluations.
 
         This callback is invoked by the ensemble optimizer before and after
@@ -86,9 +85,10 @@ class EnsembleOptimizer:
     classes are recommended for greater flexibility and ease of use.
     """
 
-    def __init__(
+    def __init__(  # noqa: PLR0913
         self,
         enopt_config: EnOptConfig,
+        transforms: OptModelTransforms | None,
         ensemble_evaluator: EnsembleEvaluator,
         plugin_manager: PluginManager,
         signal_evaluation: SignalEvaluationCallback | None = None,
@@ -103,11 +103,14 @@ class EnsembleOptimizer:
         The `EnsembleOptimizer` needs the following to define a single
         optimization run:
 
-        1.  An [`EnOptConfig`][ropt.config.enopt.EnOptConfig] object: This
-            contains all configuration settings for the optimization.
-        2.  An [`EnsembleEvaluator`][ropt.ensemble_evaluator.EnsembleEvaluator]
+        1.  An [`EnOptConfig`][ropt.config.EnOptConfig] object: This contains
+            all configuration settings for the optimization.
+        2.  A [`OptModelTransforms`][ropt.transforms.OptModelTransforms] object:
+            This handles the transforms to apply to the variables, objectives and
+            constraints.
+        3.  An [`EnsembleEvaluator`][ropt.ensemble_evaluator.EnsembleEvaluator]
             object: This object is responsible for evaluating functions.
-        3.  A [`PluginManager`][ropt.plugins.PluginManager] object: This object
+        4.  A [`PluginManager`][ropt.plugins.PluginManager] object: This object
             provides access to optimizer plugins.
 
         Additionally, two optional callbacks can be provided to extend the
@@ -121,14 +124,21 @@ class EnsembleOptimizer:
             This callback is invoked at each function evaluation to run a nested
             optimization.
 
+        The optimizer plugins are used by the ensemble optimizer to implement
+        the actual optimization process. The `EnsembleOptimizer` class provides
+        the callback function to these plugins needed (see
+        [OptimizerCallback][ropt.optimization.OptimizerCallback])
+
         Args:
             enopt_config:       The ensemble optimization configuration.
+            transforms:         The transforms to apply to the model.
             ensemble_evaluator: The evaluator for function evaluations.
             plugin_manager:     The plugin manager.
             signal_evaluation:  Optional callback to signal evaluations.
             nested_optimizer:   Optional callback for nested optimizations.
         """
         self._enopt_config = enopt_config
+        self._transforms = transforms
         self._function_evaluator = ensemble_evaluator
         self._plugin_manager = plugin_manager
         self._signal_evaluation = signal_evaluation
@@ -143,9 +153,19 @@ class EnsembleOptimizer:
 
         # Whether NaN values are allowed:
         self._allow_nan = False
-        self._optimizer: Optimizer = self._plugin_manager.get_plugin(
+
+        plugin = self._plugin_manager.get_plugin(
             "optimizer", method=self._enopt_config.optimizer.method
-        ).create(self._enopt_config, self._optimizer_callback)
+        )
+
+        # Validate the optimizer options:
+        plugin.validate_options(
+            self._enopt_config.optimizer.method, self._enopt_config.optimizer.options
+        )
+
+        self._optimizer: Optimizer = plugin.create(
+            self._enopt_config, self._optimizer_callback
+        )
         self._allow_nan = self._optimizer.allow_nan
 
         # Optional redirection of standard output:
@@ -166,7 +186,7 @@ class EnsembleOptimizer:
         """
         return self._optimizer.is_parallel
 
-    def start(self, variables: NDArray[np.float64]) -> OptimizerExitCode:
+    def start(self, variables: NDArray[np.float64]) -> ExitCode:
         """Start the optimization process.
 
         This method initiates the optimization process using the provided
@@ -177,15 +197,15 @@ class EnsembleOptimizer:
             variables: The initial variables for the optimization.
 
         Returns:
-            An [`OptimizerExitCode`][ropt.enums.OptimizerExitCode] indicating
-            the reason for termination.
+            An [`ExitCode`][ropt.enums.ExitCode] indicating the reason for
+            termination.
         """
         self._fixed_variables = variables.copy()
-        exit_code = OptimizerExitCode.OPTIMIZER_STEP_FINISHED
+        exit_code = ExitCode.OPTIMIZER_STEP_FINISHED
         try:
             with self._redirector.start():
                 self._optimizer.start(variables)
-        except OptimizationAborted as exc:
+        except StepAborted as exc:
             exit_code = exc.exit_code
         return exit_code
 
@@ -195,7 +215,7 @@ class EnsembleOptimizer:
         *,
         return_functions: bool,
         return_gradients: bool,
-    ) -> tuple[NDArray[np.float64], NDArray[np.float64]]:
+    ) -> OptimizerCallbackResult:
         assert return_functions or return_gradients
 
         self._check_stopping_criteria()
@@ -213,11 +233,9 @@ class EnsembleOptimizer:
                 variables = variables[0, ...]
             nested_results, aborted = self._nested_optimizer(variables)
             if aborted:
-                raise OptimizationAborted(exit_code=OptimizerExitCode.USER_ABORT)
+                raise StepAborted(exit_code=ExitCode.USER_ABORT)
             if nested_results is None:
-                raise OptimizationAborted(
-                    exit_code=OptimizerExitCode.NESTED_OPTIMIZER_FAILED
-                )
+                raise StepAborted(exit_code=ExitCode.NESTED_OPTIMIZER_FAILED)
             variables = nested_results.evaluations.variables
             self._fixed_variables = variables.copy()
 
@@ -227,7 +245,7 @@ class EnsembleOptimizer:
             compute_gradients=return_gradients,
         )
 
-        functions = np.array([])
+        functions = None
         if return_functions:
             # Functions might be parallelized hence we need potentially to
             # process a list of function results:
@@ -241,7 +259,7 @@ class EnsembleOptimizer:
                 np.vstack(functions_list) if variables.ndim > 1 else functions_list[0]
             )
 
-        gradients = np.array([])
+        gradients = None
         if return_gradients:
             # Gradients cannot be parallelized, there is at most one gradient:
             gradients = self._gradients_from_results(
@@ -255,31 +273,35 @@ class EnsembleOptimizer:
 
         self._completed_batches += 1
 
-        return functions, gradients
+        return OptimizerCallbackResult(
+            functions=functions,
+            gradients=gradients,
+            nonlinear_constraint_bounds=_get_nonlinear_constraint_bounds(
+                self._enopt_config, self._transforms
+            ),
+        )
 
     def _get_completed_variables(
         self, variables: NDArray[np.float64]
     ) -> NDArray[np.float64]:
         mask = self._enopt_config.variables.mask
-        if mask is not None:
-            if variables.ndim > 1:
-                tmp_variables = np.repeat(
-                    self._fixed_variables[np.newaxis, :], variables.shape[0], axis=0
-                )
-                tmp_variables[:, mask] = variables
-            else:
-                tmp_variables = self._fixed_variables.copy()
-                tmp_variables[mask] = variables
-            return tmp_variables
-        return variables.copy()
+        if variables.ndim > 1:
+            tmp_variables = np.repeat(
+                self._fixed_variables[np.newaxis, :], variables.shape[0], axis=0
+            )
+            tmp_variables[:, mask] = variables
+        else:
+            tmp_variables = self._fixed_variables.copy()
+            tmp_variables[mask] = variables
+        return tmp_variables
 
     def _check_stopping_criteria(self) -> None:
         max_functions = self._enopt_config.optimizer.max_functions
         if max_functions is not None and self._completed_functions >= max_functions:
-            raise OptimizationAborted(exit_code=OptimizerExitCode.MAX_FUNCTIONS_REACHED)
+            raise StepAborted(exit_code=ExitCode.MAX_FUNCTIONS_REACHED)
         max_batches = self._enopt_config.optimizer.max_batches
         if max_batches is not None and self._completed_batches >= max_batches:
-            raise OptimizationAborted(exit_code=OptimizerExitCode.MAX_BATCHES_REACHED)
+            raise StepAborted(exit_code=ExitCode.MAX_BATCHES_REACHED)
 
     def _run_evaluations(
         self,
@@ -307,7 +329,7 @@ class EnsembleOptimizer:
                 self._enopt_config.realizations.realization_min_success < 1
                 and not self._allow_nan
             )
-            exit_code: OptimizerExitCode | None = None
+            exit_code: ExitCode | None = None
             for result in results:
                 assert isinstance(result, FunctionResults | GradientResults)
                 if (
@@ -320,14 +342,14 @@ class EnsembleOptimizer:
                         and np.all(result.realizations.failed_realizations)
                     )
                 ):
-                    exit_code = OptimizerExitCode.TOO_FEW_REALIZATIONS
+                    exit_code = ExitCode.TOO_FEW_REALIZATIONS
                     break
 
             if self._signal_evaluation:
                 self._signal_evaluation(results)
 
             if exit_code is not None:
-                raise OptimizationAborted(exit_code=exit_code)
+                raise StepAborted(exit_code=exit_code)
 
         return results
 
@@ -422,3 +444,19 @@ class _Redirector:
                 os.dup2(self._new_stderr, 2)
         else:
             yield
+
+
+def _get_nonlinear_constraint_bounds(
+    config: EnOptConfig, transforms: OptModelTransforms | None = None
+) -> tuple[NDArray[np.float64], NDArray[np.float64]] | None:
+    if config.nonlinear_constraints is None:
+        return None
+    if transforms is not None and transforms.nonlinear_constraints is not None:
+        return transforms.nonlinear_constraints.bounds_to_optimizer(
+            config.nonlinear_constraints.lower_bounds,
+            config.nonlinear_constraints.upper_bounds,
+        )
+    return (
+        config.nonlinear_constraints.lower_bounds,
+        config.nonlinear_constraints.upper_bounds,
+    )
