@@ -22,11 +22,12 @@ from ropt.config.options import OptionsSchemaModel
 from ropt.enums import VariableType
 from ropt.plugins.optimizer.utils import validate_supported_constraints
 
-from .base import Optimizer, OptimizerCallback, OptimizerPlugin
+from .base import Optimizer, OptimizerPlugin
 from .utils import NormalizedConstraints, get_masked_linear_constraints
 
 if TYPE_CHECKING:
-    from ropt.config.enopt import EnOptConfig
+    from ropt.config import EnOptConfig
+    from ropt.optimization import OptimizerCallback
 
 _SUPPORTED_METHODS: Final[set[str]] = {
     name.lower()
@@ -43,6 +44,7 @@ _SUPPORTED_METHODS: Final[set[str]] = {
         "differential_evolution",
     )
 }
+_DEFAULT_METHOD: Final = "slsqp"
 
 
 # Categorize the methods by the types of constraint they support or require.
@@ -93,12 +95,12 @@ class SciPyOptimizer(Optimizer):
     module, enabling their use within `ropt`.
 
     To select an optimizer, set the `method` field within the
-    [`optimizer`][ropt.config.enopt.OptimizerConfig] section of the
-    [`EnOptConfig`][ropt.config.enopt.EnOptConfig] configuration object to the
-    desired algorithm's name. Most methods support the general options defined
-    in the [`EnOptConfig`][ropt.config.enopt.EnOptConfig] object. For
-    algorithm-specific options, use the `options` dictionary within the
-    [`optimizer`][ropt.config.enopt.OptimizerConfig] section.
+    [`optimizer`][ropt.config.OptimizerConfig] section of the
+    [`EnOptConfig`][ropt.config.EnOptConfig] configuration object to the desired
+    algorithm's name. Most methods support the general options defined in the
+    [`EnOptConfig`][ropt.config.EnOptConfig] object. For algorithm-specific
+    options, use the `options` dictionary within the
+    [`optimizer`][ropt.config.OptimizerConfig] section.
 
     The table below lists the included methods together with the method-specific
     options that are supported. Click on the method name to consult the
@@ -133,7 +135,7 @@ class SciPyOptimizer(Optimizer):
         self._config = config
         _, _, self._method = self._config.optimizer.method.lower().rpartition("/")
         if self._method == "default":
-            self._method = "slsqp"
+            self._method = _DEFAULT_METHOD
         if self._method not in _SUPPORTED_METHODS:
             msg = f"SciPy optimizer algorithm {self._method} is not supported"
             raise NotImplementedError(msg)
@@ -143,8 +145,6 @@ class SciPyOptimizer(Optimizer):
             self._supported_constraints,
             self._required_constraints,
         )
-        self._bounds = self._initialize_bounds()
-        self._constraints = self._initialize_constraints()
         self._options = self._parse_options()
         self._parallel = (
             self._config.optimizer.parallel and self._method == "differential_evolution"
@@ -165,8 +165,8 @@ class SciPyOptimizer(Optimizer):
         self._cached_function = None
         self._cached_gradient = None
 
-        if self._config.variables.mask is not None:
-            initial_values = initial_values[..., self._config.variables.mask]
+        self._bounds = self._initialize_bounds()
+        self._constraints = self._initialize_constraints(initial_values)
 
         if self._method == "differential_evolution":
             if self._parallel:
@@ -174,7 +174,7 @@ class SciPyOptimizer(Optimizer):
                 self._options["workers"] = 1
             differential_evolution(
                 func=self._function,
-                x0=initial_values,
+                x0=initial_values[self._config.variables.mask],
                 bounds=self._bounds,
                 constraints=self._constraints,
                 polish=False,
@@ -184,7 +184,7 @@ class SciPyOptimizer(Optimizer):
         else:
             minimize(
                 fun=self._function,
-                x0=initial_values,
+                x0=initial_values[self._config.variables.mask],
                 tol=self._config.optimizer.tolerance,
                 method=self._method,
                 bounds=self._bounds,
@@ -218,20 +218,21 @@ class SciPyOptimizer(Optimizer):
             np.isfinite(self._config.variables.lower_bounds).any()
             or np.isfinite(self._config.variables.upper_bounds).any()
         ):
-            lower_bounds = self._config.variables.lower_bounds
-            upper_bounds = self._config.variables.upper_bounds
-            if self._config.variables.mask is not None:
-                lower_bounds = lower_bounds[self._config.variables.mask]
-                upper_bounds = upper_bounds[self._config.variables.mask]
+            lower_bounds = self._config.variables.lower_bounds[
+                self._config.variables.mask
+            ]
+            upper_bounds = self._config.variables.upper_bounds[
+                self._config.variables.mask
+            ]
             return Bounds(lower_bounds, upper_bounds)
         return None
 
     def _get_constraint_bounds(
-        self,
+        self, nonlinear_bounds: tuple[NDArray[np.float64], NDArray[np.float64]] | None
     ) -> tuple[NDArray[np.float64], NDArray[np.float64]] | None:
         bounds = []
-        if self._config.nonlinear_constraints is not None:
-            bounds.append(self._config.nonlinear_constraints.get_bounds())
+        if nonlinear_bounds is not None:
+            bounds.append(nonlinear_bounds)
         if (
             self._method != "differential_evolution"
             and self._linear_constraint_bounds is not None
@@ -244,7 +245,7 @@ class SciPyOptimizer(Optimizer):
         return None
 
     def _initialize_constraints(
-        self,
+        self, initial_values: NDArray[np.float64]
     ) -> list[dict[str, _ConstraintType] | NonlinearConstraint | LinearConstraint]:
         self._normalized_constraints = None
 
@@ -253,13 +254,21 @@ class SciPyOptimizer(Optimizer):
             tuple[NDArray[np.float64], NDArray[np.float64]] | None
         ) = None
         if self._config.linear_constraints is not None:
-            lin_coef, lin_lower, lin_upper = get_masked_linear_constraints(self._config)
+            lin_coef, lin_lower, lin_upper = get_masked_linear_constraints(
+                self._config, initial_values
+            )
             self._linear_constraint_bounds = (lin_lower, lin_upper)
-
-        if (bounds := self._get_constraint_bounds()) is not None:
+        nonlinear_bounds = (
+            None
+            if self._config.nonlinear_constraints is None
+            else (
+                self._config.nonlinear_constraints.lower_bounds,
+                self._config.nonlinear_constraints.upper_bounds,
+            )
+        )
+        if (bounds := self._get_constraint_bounds(nonlinear_bounds)) is not None:
             self._normalized_constraints = NormalizedConstraints()
             self._normalized_constraints.set_bounds(*bounds)
-
         if self._method == "differential_evolution":
             return self._initialize_constraints_object(lin_coef, lin_lower, lin_upper)
 
@@ -466,37 +475,48 @@ class SciPyOptimizer(Optimizer):
             and compute_gradients
             and self._config.gradient.evaluation_policy == "separate"
         ):
-            new_function, _ = self._optimizer_callback(
+            callback_result = self._optimizer_callback(
                 variables,
                 return_functions=True,
                 return_gradients=False,
             )
-            _, new_gradient = self._optimizer_callback(
+            new_function = callback_result.functions
+            callback_result = self._optimizer_callback(
                 variables,
                 return_functions=False,
                 return_gradients=True,
             )
+            new_gradient = callback_result.gradients
         else:
-            new_function, new_gradient = self._optimizer_callback(
+            callback_result = self._optimizer_callback(
                 variables,
                 return_functions=compute_functions,
                 return_gradients=compute_gradients,
             )
+            new_function = callback_result.functions
+            new_gradient = callback_result.gradients
 
         # The optimizer callback may change non-linear constraint bounds:
-        if self._normalized_constraints is not None:
-            bounds = self._get_constraint_bounds()
+        if (
+            self._normalized_constraints is not None
+            and callback_result.nonlinear_constraint_bounds is not None
+        ):
+            bounds = self._get_constraint_bounds(
+                callback_result.nonlinear_constraint_bounds
+            )
             assert bounds is not None
             self._normalized_constraints.set_bounds(*bounds)
 
-        if self.allow_nan:
+        if self.allow_nan and new_function is not None:
             new_function = np.where(np.isnan(new_function), np.inf, new_function)
         return new_function, new_gradient
 
     def _parse_options(self) -> dict[str, Any]:
-        if not isinstance(self._config.optimizer.options, dict):
-            return {}
-        options = copy.deepcopy(self._config.optimizer.options)
+        options = (
+            copy.deepcopy(self._config.optimizer.options)
+            if isinstance(self._config.optimizer.options, dict)
+            else {}
+        )
         # The maximum number of iterations is passed as an option to ropt.
         # Setting maxiter directly as an entry in the options dict will also
         # work, but iterations will override it.
@@ -510,11 +530,7 @@ class SciPyOptimizer(Optimizer):
         if self._config.optimizer.output_dir is not None:
             options["disp"] = True
 
-        if (
-            self._method == "differential_evolution"
-            and self._config.variables.types is not None
-            and "integrality" not in options
-        ):
+        if self._method == "differential_evolution" and "integrality" not in options:
             options["integrality"] = (
                 self._config.variables.types == VariableType.INTEGER
             )
@@ -558,8 +574,12 @@ class SciPyOptimizerPlugin(OptimizerPlugin):
         # noqa
         """
         if options is not None:
+            if not isinstance(options, dict):
+                msg = "SciPy optimizer options must be a dictionary"
+                raise ValueError(msg)
+            *_, method = method.rpartition("/")
             OptionsSchemaModel.model_validate(_OPTIONS_SCHEMA).get_options_model(
-                method
+                _DEFAULT_METHOD if method == "default" else method
             ).model_validate(options)
 
 
@@ -701,7 +721,7 @@ _OPTIONS_SCHEMA: dict[str, Any] = {
                 "tol": float,
                 "mutation": float | tuple[float, float],
                 "recombination": float,
-                "seed": int,
+                "rng": int,
                 "polish": bool,
                 "init": Literal["latinhypercube", "sobol", "haltonrandom"],
                 "atol": float,
