@@ -9,7 +9,7 @@ from pathlib import Path
 import numpy as np
 from numpy.typing import NDArray
 
-from ropt.config.enopt import EnOptConfig
+from ropt.config import EnOptConfig
 
 _MESSAGES = {
     "bounds": "bound constraints",
@@ -319,7 +319,6 @@ class NormalizedConstraints:
             or not np.array_equal(self._upper_bounds, upper_bounds)
         ):
             is_eq = self._is_eq
-
             self._lower_bounds = lower_bounds.copy()
             self._upper_bounds = upper_bounds.copy()
             self._is_eq = []
@@ -479,7 +478,7 @@ class NormalizedConstraints:
 
 
 def get_masked_linear_constraints(
-    config: EnOptConfig,
+    config: EnOptConfig, initial_values: NDArray[np.float64]
 ) -> tuple[NDArray[np.float64], NDArray[np.float64], NDArray[np.float64]]:
     """Adjust linear constraints based on a variable mask.
 
@@ -500,8 +499,9 @@ def get_masked_linear_constraints(
     are removed entirely, as they become trivial constants.
 
     Args:
-        config: The [`EnOptConfig`][ropt.config.enopt.EnOptConfig] object
-                containing the variable mask and linear constraints.
+        config:         The [`EnOptConfig`][ropt.config.EnOptConfig] object
+                        containing the variable mask and linear constraints.
+        initial_values: The initial values to use.
 
     Returns:
         The adjusted coefficients and bounds.
@@ -511,15 +511,13 @@ def get_masked_linear_constraints(
     coefficients = config.linear_constraints.coefficients
     lower_bounds = config.linear_constraints.lower_bounds
     upper_bounds = config.linear_constraints.upper_bounds
-    if mask is not None:
+    if not np.all(mask):
         # Keep rows that only contain non-zero values for the active variables:
         keep_rows = np.all(coefficients[:, ~mask] == 0, axis=1)
         coefficients = coefficients[keep_rows, :]
         lower_bounds = lower_bounds[keep_rows]
         upper_bounds = upper_bounds[keep_rows]
-        offsets = np.matmul(
-            coefficients[:, ~mask], config.variables.initial_values[~mask]
-        )
+        offsets = np.matmul(coefficients[:, ~mask], initial_values[~mask])
         coefficients = coefficients[:, mask]
     else:
         offsets = 0
