@@ -6,9 +6,9 @@ import numpy as np
 from numpy.typing import NDArray
 from pydantic import BaseModel, ConfigDict, Field, NonNegativeInt
 
-from ropt.config.enopt import EnOptConfig
-from ropt.enums import OptimizerExitCode
-from ropt.exceptions import ConfigError, OptimizationAborted
+from ropt.config import EnOptConfig
+from ropt.enums import ExitCode
+from ropt.exceptions import StepAborted
 
 from .base import RealizationFilter, RealizationFilterPlugin
 
@@ -19,6 +19,7 @@ class _ConfigBaseModel(BaseModel):
         extra="forbid",
         str_min_length=1,
         str_strip_whitespace=True,
+        frozen=True,
     )
 
 
@@ -31,9 +32,9 @@ class SortObjectiveOptions(_ConfigBaseModel):
     How it works:
 
     1. A weighted sum is calculated for each realization using the objective
-       values specified by the `sort` indices and the corresponding weights
-       from the main [`EnOptConfig`][ropt.config.enopt.EnOptConfig]. If only
-       one objective index is provided in `sort`, no weighting is applied.
+       values specified by the `sort` indices and the corresponding weights from
+       the main [`EnOptConfig`][ropt.config.EnOptConfig]. If only one objective
+       index is provided in `sort`, no weighting is applied.
     2. Realizations are sorted based on this calculated value (ascending).
     3. Realizations whose rank falls within the range [`first`, `last`]
        (inclusive) are selected.
@@ -48,7 +49,7 @@ class SortObjectiveOptions(_ConfigBaseModel):
         last:  The ending rank (0-based index) of realizations to select after sorting.
     """
 
-    sort: list[NonNegativeInt]
+    sort: tuple[NonNegativeInt]
     first: NonNegativeInt
     last: NonNegativeInt
 
@@ -94,7 +95,7 @@ class CVaRObjectiveOptions(_ConfigBaseModel):
 
     1. A weighted sum is calculated for each realization using the objective
        values specified by the `sort` indices and the corresponding weights
-       from the main [`EnOptConfig`][ropt.config.enopt.EnOptConfig]. If only
+       from the main [`EnOptConfig`][ropt.config.EnOptConfig]. If only
        one objective index is provided in `sort`, no weighting is applied.
     2. Realizations are conceptually sorted based on this calculated value
        (ascending, assuming minimization).
@@ -113,7 +114,7 @@ class CVaRObjectiveOptions(_ConfigBaseModel):
                     worst realizations to consider. Defaults to 0.5.
     """
 
-    sort: list[NonNegativeInt]
+    sort: tuple[NonNegativeInt]
     percentile: Annotated[float, Field(gt=0.0, le=1.0)] = 0.5
 
 
@@ -164,8 +165,8 @@ class DefaultRealizationFilter(RealizationFilter):
     This class provides several methods for calculating realization weights based
     on objective or constraint values. The specific method and its parameters
     are configured via the
-    [`RealizationFilterConfig`][ropt.config.enopt.RealizationFilterConfig]
-    in the main [`EnOptConfig`][ropt.config.enopt.EnOptConfig].
+    [`RealizationFilterConfig`][ropt.config.RealizationFilterConfig]
+    in the main [`EnOptConfig`][ropt.config.EnOptConfig].
 
     **Supported Methods:**
 
@@ -236,7 +237,7 @@ class DefaultRealizationFilter(RealizationFilter):
                 self._filter_options = CVaRConstraintOptions.model_validate(options)
             case _:
                 msg = f"Realization filter not supported: {self._method}"
-                raise ConfigError(msg)
+                raise ValueError(msg)
 
     def get_realization_weights(  # D107
         self,
@@ -263,10 +264,10 @@ class DefaultRealizationFilter(RealizationFilter):
                 weights = self._cvar_constraint(constraints)
             case _:
                 msg = f"Realization filter not supported: {self._method}"
-                raise ConfigError(msg)
+                raise ValueError(msg)
 
         if not np.any(weights > 0):
-            raise OptimizationAborted(exit_code=OptimizerExitCode.TOO_FEW_REALIZATIONS)
+            raise StepAborted(exit_code=ExitCode.TOO_FEW_REALIZATIONS)
 
         return weights
 
@@ -276,11 +277,11 @@ class DefaultRealizationFilter(RealizationFilter):
         realizations = self._enopt_config.realizations.weights.size
         msg = f"Invalid range of realizations: [{options.first}, {options.last}]"
         if options.first < 0 or options.first >= realizations:
-            raise ConfigError(msg)
+            raise ValueError(msg)
         if options.last < 0 or options.last >= realizations:
-            raise ConfigError(msg)
+            raise ValueError(msg)
         if options.last < options.first:
-            raise ConfigError(msg)
+            raise ValueError(msg)
 
     def _sort_objectives(self, objectives: NDArray[np.float64]) -> NDArray[np.float64]:
         assert isinstance(self._filter_options, SortObjectiveOptions)
