@@ -4,11 +4,13 @@ from typing import TYPE_CHECKING, Any
 
 import numpy as np
 import pytest
+from pydantic import ValidationError
 
-from ropt.config.enopt import EnOptConfig
-from ropt.config.enopt.constants import DEFAULT_SEED
-from ropt.enums import EventType, OptimizerExitCode
+from ropt.config import EnOptConfig
+from ropt.config.constants import DEFAULT_SEED
+from ropt.enums import EventType, ExitCode
 from ropt.plan import BasicOptimizer
+from ropt.plugins._manager import PluginManager
 from ropt.results import FunctionResults, GradientResults, Results
 from ropt.transforms import OptModelTransforms, VariableScaler
 from ropt.transforms.base import NonLinearConstraintTransform, ObjectiveTransform
@@ -18,27 +20,59 @@ if TYPE_CHECKING:
 
     from ropt.plan import Event
 
+_SLSQP = "slsqp"
+_DIFFERENTIAL_EVOLUTION = "differential_evolution"
+
+pytestmark = [
+    pytest.mark.parametrize(
+        "external", ["", pytest.param("external/", marks=pytest.mark.external)]
+    )
+]
+
+initial_values = [0.0, 0.0, 0.1]
+
 
 @pytest.fixture(name="enopt_config")
 def enopt_config_fixture() -> dict[str, Any]:
     return {
         "optimizer": {
+            "method": _SLSQP,
+            "max_iterations": 15,
             "tolerance": 1e-5,
             "max_functions": 20,
         },
         "variables": {
-            "initial_values": [0.0, 0.0, 0.1],
+            "variable_count": len(initial_values),
+            "perturbation_magnitudes": 0.01,
         },
         "objectives": {
             "weights": [0.75, 0.25],
         },
-        "gradient": {
-            "perturbation_magnitudes": 0.01,
-        },
     }
 
 
-def test_max_functions_exceeded(enopt_config: Any, evaluator: Any) -> None:
+def test_basic_run(enopt_config: Any, evaluator: Any, external: str) -> None:
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+    variables = BasicOptimizer(enopt_config, evaluator()).run(initial_values).variables
+    assert variables is not None
+    assert np.allclose(variables, [0, 0, 0.5], atol=0.02)
+
+
+def test_invalid_options(enopt_config: Any, external: str) -> None:
+    enopt_config["optimizer"]["options"] = {"ftol": 0.1, "foo": 1}
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+
+    method = enopt_config["optimizer"]["method"]
+    plugin = PluginManager().get_plugin("optimizer", method)
+    with pytest.raises(
+        ValidationError, match=r"Unknown or unsupported option\(s\): `foo`"
+    ):
+        plugin.validate_options(method, enopt_config["optimizer"]["options"])
+
+
+def test_max_functions_exceeded(
+    enopt_config: Any, evaluator: Any, external: str
+) -> None:
     last_evaluation = 0
 
     def track_results(_: tuple[Results, ...]) -> None:
@@ -47,15 +81,16 @@ def test_max_functions_exceeded(enopt_config: Any, evaluator: Any) -> None:
 
     max_functions = 2
     enopt_config["optimizer"]["max_functions"] = max_functions
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
     optimizer = BasicOptimizer(enopt_config, evaluator()).set_results_callback(
         track_results
     )
-    optimizer.run()
+    optimizer.run(initial_values)
     assert last_evaluation == max_functions + 1
-    assert optimizer.exit_code == OptimizerExitCode.MAX_FUNCTIONS_REACHED
+    assert optimizer.exit_code == ExitCode.MAX_FUNCTIONS_REACHED
 
 
-def test_max_batches_exceeded(enopt_config: Any, evaluator: Any) -> None:
+def test_max_batches_exceeded(enopt_config: Any, evaluator: Any, external: str) -> None:
     last_evaluation = 0
 
     def track_results(_: tuple[Results, ...]) -> None:
@@ -64,15 +99,18 @@ def test_max_batches_exceeded(enopt_config: Any, evaluator: Any) -> None:
 
     max_batches = 2
     enopt_config["optimizer"]["max_batches"] = max_batches
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
     optimizer = BasicOptimizer(enopt_config, evaluator()).set_results_callback(
         track_results
     )
-    optimizer.run()
+    optimizer.run(initial_values)
     assert last_evaluation == max_batches
-    assert optimizer.exit_code == OptimizerExitCode.MAX_BATCHES_REACHED
+    assert optimizer.exit_code == ExitCode.MAX_BATCHES_REACHED
 
 
-def test_max_functions_not_exceeded(enopt_config: Any, evaluator: Any) -> None:
+def test_max_functions_not_exceeded(
+    enopt_config: Any, evaluator: Any, external: str
+) -> None:
     last_evaluation = 0
 
     def track_results(_: tuple[Results, ...]) -> None:
@@ -81,43 +119,67 @@ def test_max_functions_not_exceeded(enopt_config: Any, evaluator: Any) -> None:
 
     max_functions = 100
     enopt_config["optimizer"]["max_functions"] = max_functions
-    enopt_config["gradient"]["evaluation_policy"] = "separate"
+    enopt_config["gradient"] = {"evaluation_policy": "separate"}
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
     optimizer = BasicOptimizer(enopt_config, evaluator()).set_results_callback(
         track_results
     )
-    optimizer.run()
+    optimizer.run(initial_values)
     assert last_evaluation + 1 < 2 * max_functions
-    assert optimizer.exit_code == OptimizerExitCode.OPTIMIZER_STEP_FINISHED
+    assert optimizer.exit_code == ExitCode.OPTIMIZER_STEP_FINISHED
 
 
-def test_failed_realizations(enopt_config: Any, evaluator: Any) -> None:
+def test_failed_realizations(enopt_config: Any, evaluator: Any, external: str) -> None:
     def _observer(results: tuple[Results, ...]) -> None:
         assert isinstance(results[0], FunctionResults)
         assert results[0].functions is None
 
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+
     functions = [lambda _0, _1: np.array(1.0), lambda _0, _1: np.array(np.nan)]
     optimizer = BasicOptimizer(enopt_config, evaluator(functions)).set_results_callback(
         _observer
     )
-    optimizer.run()
-    assert optimizer.exit_code == OptimizerExitCode.TOO_FEW_REALIZATIONS
+    optimizer.run(initial_values)
+    assert optimizer.exit_code == ExitCode.TOO_FEW_REALIZATIONS
+
+
+def test_failed_realizations_constraints(
+    enopt_config: Any, evaluator: Any, test_functions: Any, external: str
+) -> None:
+    def _observer(results: tuple[Results, ...]) -> None:
+        assert isinstance(results[0], FunctionResults)
+        assert results[0].functions is None
+
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+    enopt_config["nonlinear_constraints"] = {
+        "lower_bounds": 0.0,
+        "upper_bounds": 0.4,
+    }
+
+    functions = (*test_functions, lambda _0, _1: np.array(np.nan))
+
+    optimizer = BasicOptimizer(enopt_config, evaluator(functions)).set_results_callback(
+        _observer
+    )
+    optimizer.run(initial_values)
+    assert optimizer.exit_code == ExitCode.TOO_FEW_REALIZATIONS
 
 
 def test_all_failed_realizations_not_supported(
-    enopt_config: Any, evaluator: Any
+    enopt_config: Any, evaluator: Any, external: str
 ) -> None:
     enopt_config["realizations"] = {"realization_min_success": 0}
 
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+
     functions = [lambda _0, _1: np.array(1.0), lambda _0, _1: np.array(np.nan)]
-    optimizer = BasicOptimizer(
-        enopt_config,
-        evaluator(functions),
-    )
-    optimizer.run()
-    assert optimizer.exit_code == OptimizerExitCode.TOO_FEW_REALIZATIONS
+    optimizer = BasicOptimizer(enopt_config, evaluator(functions))
+    optimizer.run(initial_values)
+    assert optimizer.exit_code == ExitCode.TOO_FEW_REALIZATIONS
 
 
-def test_user_abort(enopt_config: Any, evaluator: Any) -> None:
+def test_user_abort(enopt_config: Any, evaluator: Any, external: str) -> None:
     last_evaluation = 0
 
     def _abort() -> bool:
@@ -128,24 +190,37 @@ def test_user_abort(enopt_config: Any, evaluator: Any) -> None:
         last_evaluation += 1
         return False
 
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+
     optimizer = BasicOptimizer(enopt_config, evaluator()).set_abort_callback(_abort)
-    optimizer.run()
+    optimizer.run(initial_values)
     assert optimizer.results is not None
     assert last_evaluation == 1
-    assert optimizer.exit_code == OptimizerExitCode.USER_ABORT
+    assert optimizer.exit_code == ExitCode.USER_ABORT
 
 
-def test_single_perturbation(enopt_config: Any, evaluator: Any) -> None:
+def test_single_perturbation(enopt_config: Any, evaluator: Any, external: str) -> None:
     enopt_config["gradient"] = {
         "number_of_perturbations": 1,
         "merge_realizations": True,
     }
+
     enopt_config["realizations"] = {"weights": 5 * [1]}
-    variables = BasicOptimizer(enopt_config, evaluator()).run().variables
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+
+    variables = BasicOptimizer(enopt_config, evaluator()).run(initial_values).variables
     assert variables is not None
     assert np.allclose(variables, [0.0, 0.0, 0.5], atol=0.02)
 
 
+def test_external_error(enopt_config: Any, evaluator: Any, external: str) -> None:
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+    enopt_config["optimizer"]["options"] = {"ftol": "foo"}
+    err = "Input should be a valid number, unable to parse string as a number"
+    with pytest.raises(ValueError, match=err):
+        BasicOptimizer(enopt_config, evaluator()).run(initial_values)
+
+
 class ObjectiveScaler(ObjectiveTransform):
     def __init__(self, scales: ArrayLike) -> None:
         self._scales = np.asarray(scales, dtype=np.float64)
@@ -164,10 +239,9 @@ class ObjectiveScaler(ObjectiveTransform):
 
 
 def test_objective_with_scaler(
-    enopt_config: Any, evaluator: Any, test_functions: Any
+    enopt_config: Any, evaluator: Any, test_functions: Any, external: str
 ) -> None:
-    config = EnOptConfig.model_validate(enopt_config)
-    results1 = BasicOptimizer(enopt_config, evaluator()).run().results
+    results1 = BasicOptimizer(enopt_config, evaluator()).run(initial_values).results
     assert results1 is not None
     assert results1.functions is not None
     variables1 = results1.evaluations.variables
@@ -175,17 +249,18 @@ def test_objective_with_scaler(
     assert np.allclose(variables1, [0.0, 0.0, 0.5], atol=0.02)
     assert np.allclose(objectives1, [0.5, 4.5], atol=0.02)
 
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+
     def function1(variables: NDArray[np.float64], _: Any) -> float:
         return float(test_functions[0](variables, None))
 
     def function2(variables: NDArray[np.float64], _: Any) -> float:
         return float(test_functions[1](variables, None))
 
-    init1 = test_functions[1](config.variables.initial_values, None)
+    init1 = test_functions[1](initial_values, None)
     transforms = OptModelTransforms(
         objectives=ObjectiveScaler(np.array([init1, init1]))
     )
-    config = EnOptConfig.model_validate(enopt_config, context=transforms)
 
     checked = False
 
@@ -198,20 +273,18 @@ def test_objective_with_scaler(
                 assert item.functions is not None
                 assert item.functions.objectives is not None
                 assert np.allclose(item.functions.objectives[-1], 1.0)
-                assert config.transforms is not None
-                transformed = item.transform_from_optimizer(config.transforms)
+                transformed = item.transform_from_optimizer(event.data["transforms"])
                 assert transformed.functions is not None
                 assert transformed.functions.objectives is not None
                 assert np.allclose(transformed.functions.objectives[-1], init1)
 
     optimizer = BasicOptimizer(
-        EnOptConfig.model_validate(enopt_config, context=transforms),
-        evaluator([function1, function2]),
+        enopt_config, evaluator([function1, function2]), transforms=transforms
     )
     optimizer._observers.append(  # noqa: SLF001
         (EventType.FINISHED_EVALUATION, check_value)
     )
-    results2 = optimizer.run().results
+    results2 = optimizer.run(initial_values).results
     assert results2 is not None
     assert np.allclose(results2.evaluations.variables, variables1, atol=0.02)
     assert results2.functions is not None
@@ -219,10 +292,11 @@ def test_objective_with_scaler(
 
 
 def test_objective_with_lazy_scaler(
-    enopt_config: Any, evaluator: Any, test_functions: Any
+    enopt_config: Any, evaluator: Any, test_functions: Any, external: str
 ) -> None:
-    config = EnOptConfig.model_validate(enopt_config)
-    results1 = BasicOptimizer(enopt_config, evaluator()).run().results
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+
+    results1 = BasicOptimizer(enopt_config, evaluator()).run(initial_values).results
     assert results1 is not None
     assert results1.functions is not None
     variables1 = results1.evaluations.variables
@@ -232,9 +306,8 @@ def test_objective_with_lazy_scaler(
 
     objective_transform = ObjectiveScaler(np.array([1.0, 1.0]))
     transforms = OptModelTransforms(objectives=objective_transform)
-    config = EnOptConfig.model_validate(enopt_config, context=transforms)
 
-    init1 = test_functions[1](config.variables.initial_values, None)
+    init1 = test_functions[1](initial_values, None)
 
     def function1(variables: NDArray[np.float64], _: Any) -> float:
         objective_transform.set_scales([init1, init1])
@@ -254,20 +327,18 @@ def test_objective_with_lazy_scaler(
                 assert item.functions is not None
                 assert item.functions.objectives is not None
                 assert np.allclose(item.functions.objectives[-1], 1.0)
-                assert config.transforms is not None
-                transformed = item.transform_from_optimizer(config.transforms)
+                transformed = item.transform_from_optimizer(event.data["transforms"])
                 assert transformed.functions is not None
                 assert transformed.functions.objectives is not None
                 assert np.allclose(transformed.functions.objectives[-1], init1)
 
     optimizer = BasicOptimizer(
-        EnOptConfig.model_validate(enopt_config, context=transforms),
-        evaluator([function1, function2]),
+        enopt_config, evaluator([function1, function2]), transforms=transforms
     )
     optimizer._observers.append(  # noqa: SLF001
         (EventType.FINISHED_EVALUATION, check_value)
     )
-    results2 = optimizer.run().results
+    results2 = optimizer.run(initial_values).results
     assert results2 is not None
     assert np.allclose(results2.evaluations.variables, variables1, atol=0.02)
     assert results2.functions is not None
@@ -302,8 +373,10 @@ class ConstraintScaler(NonLinearConstraintTransform):
 
 
 def test_nonlinear_constraint_with_scaler(
-    enopt_config: Any, evaluator: Any, test_functions: Any
+    enopt_config: Any, evaluator: Any, test_functions: Any, external: str
 ) -> None:
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+
     enopt_config["nonlinear_constraints"] = {
         "lower_bounds": 0.0,
         "upper_bounds": 0.4,
@@ -314,21 +387,23 @@ def test_nonlinear_constraint_with_scaler(
         lambda variables, _: variables[0] + variables[2],
     )
 
-    config = EnOptConfig.model_validate(enopt_config)
-    results1 = BasicOptimizer(config, evaluator(functions)).run().results
+    results1 = (
+        BasicOptimizer(enopt_config, evaluator(functions)).run(initial_values).results
+    )
     assert results1 is not None
     assert results1.evaluations.variables[[0, 2]].sum() > 0.0 - 1e-5
     assert results1.evaluations.variables[[0, 2]].sum() < 0.4 + 1e-5
 
-    scales = np.array(
-        functions[-1](enopt_config["variables"]["initial_values"], None), ndmin=1
-    )
+    scales = np.array(functions[-1](initial_values, None), ndmin=1)
     transforms = OptModelTransforms(nonlinear_constraints=ConstraintScaler(scales))
     config = EnOptConfig.model_validate(enopt_config, context=transforms)
     assert config.nonlinear_constraints is not None
     assert config.nonlinear_constraints.upper_bounds == 0.4
-    bounds = config.nonlinear_constraints.get_bounds()
-    assert bounds is not None
+    assert transforms.nonlinear_constraints is not None
+    bounds = transforms.nonlinear_constraints.bounds_to_optimizer(
+        config.nonlinear_constraints.lower_bounds,
+        config.nonlinear_constraints.upper_bounds,
+    )
     assert bounds[1] == 0.4 / scales
 
     check = True
@@ -342,16 +417,18 @@ def test_nonlinear_constraint_with_scaler(
                 assert item.functions is not None
                 assert item.functions.constraints is not None
                 assert np.allclose(item.functions.constraints, 1.0)
-                transformed = item.transform_from_optimizer(transforms)
+                transformed = item.transform_from_optimizer(event.data["transforms"])
                 assert transformed.functions is not None
                 assert transformed.functions.constraints is not None
                 assert np.allclose(transformed.functions.constraints, scales)
 
-    optimizer = BasicOptimizer(config, evaluator(functions))
+    optimizer = BasicOptimizer(
+        enopt_config, evaluator(functions), transforms=transforms
+    )
     optimizer._observers.append(  # noqa: SLF001
         (EventType.FINISHED_EVALUATION, check_constraints)
     )
-    results2 = optimizer.run().results
+    results2 = optimizer.run(initial_values).results
     assert results2 is not None
     assert np.allclose(
         results2.evaluations.variables, results1.evaluations.variables, atol=0.02
@@ -364,8 +441,10 @@ def test_nonlinear_constraint_with_scaler(
 
 
 def test_nonlinear_constraint_with_lazy_scaler(
-    enopt_config: Any, evaluator: Any, test_functions: Any
+    enopt_config: Any, evaluator: Any, test_functions: Any, external: str
 ) -> None:
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+
     enopt_config["nonlinear_constraints"] = {
         "lower_bounds": 0.0,
         "upper_bounds": 0.4,
@@ -376,23 +455,25 @@ def test_nonlinear_constraint_with_lazy_scaler(
         lambda variables, _: variables[0] + variables[2],
     )
 
-    config = EnOptConfig.model_validate(enopt_config)
-    results1 = BasicOptimizer(config, evaluator(functions)).run().results
+    results1 = (
+        BasicOptimizer(enopt_config, evaluator(functions)).run(initial_values).results
+    )
     assert results1 is not None
     assert results1.evaluations.variables[[0, 2]].sum() > 0.0 - 1e-5
     assert results1.evaluations.variables[[0, 2]].sum() < 0.4 + 1e-5
 
-    scales = np.array(
-        functions[-1](enopt_config["variables"]["initial_values"], None), ndmin=1
-    )
+    scales = np.array(functions[-1](initial_values, None), ndmin=1)
     scaler = ConstraintScaler([1.0])
     transforms = OptModelTransforms(nonlinear_constraints=scaler)
 
     config = EnOptConfig.model_validate(enopt_config, context=transforms)
     assert config.nonlinear_constraints is not None
     assert config.nonlinear_constraints.upper_bounds == 0.4
-    bounds = config.nonlinear_constraints.get_bounds()
-    assert bounds is not None
+    assert transforms.nonlinear_constraints is not None
+    bounds = transforms.nonlinear_constraints.bounds_to_optimizer(
+        config.nonlinear_constraints.lower_bounds,
+        config.nonlinear_constraints.upper_bounds,
+    )
     assert bounds[1] == 0.4
 
     def constraint_function(variables: NDArray[np.float64], _: Any) -> float:
@@ -406,25 +487,32 @@ def test_nonlinear_constraint_with_lazy_scaler(
     def check_constraints(event: Event) -> None:
         nonlocal check
         results = event.data.get("results", ())
+        config = event.data["config"]
         for item in results:
             if isinstance(item, FunctionResults) and check:
                 check = False
                 assert config.nonlinear_constraints is not None
-                _, upper_bounds = config.nonlinear_constraints.get_bounds()
+                assert transforms.nonlinear_constraints is not None
+                _, upper_bounds = transforms.nonlinear_constraints.bounds_to_optimizer(
+                    config.nonlinear_constraints.lower_bounds,
+                    config.nonlinear_constraints.upper_bounds,
+                )
                 assert np.allclose(upper_bounds, 0.4 / scales)
                 assert item.functions is not None
                 assert item.functions.constraints is not None
                 assert np.allclose(item.functions.constraints, 1.0)
-                transformed = item.transform_from_optimizer(transforms)
+                transformed = item.transform_from_optimizer(event.data["transforms"])
                 assert transformed.functions is not None
                 assert transformed.functions.constraints is not None
                 assert np.allclose(transformed.functions.constraints, scales)
 
-    optimizer = BasicOptimizer(config, evaluator(functions))
+    optimizer = BasicOptimizer(
+        enopt_config, evaluator(functions), transforms=transforms
+    )
     optimizer._observers.append(  # noqa: SLF001
         (EventType.FINISHED_EVALUATION, check_constraints)
     )
-    results2 = optimizer.run().results
+    results2 = optimizer.run(initial_values).results
     assert results2 is not None
     assert np.allclose(
         results2.evaluations.variables, results1.evaluations.variables, atol=0.02
@@ -443,8 +531,10 @@ def test_variables_scale_with_scaler(
     evaluator: Any,
     offsets: NDArray[np.float64] | None,
     scales: NDArray[np.float64] | None,
+    external: str,
 ) -> None:
-    initial_values = np.array(enopt_config["variables"]["initial_values"])
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+
     lower_bounds = np.array([-2.0, -np.inf, -3.0])
     upper_bounds = np.array([np.inf, 1.0, 4.0])
 
@@ -454,25 +544,19 @@ def test_variables_scale_with_scaler(
 
     transforms = OptModelTransforms(variables=VariableScaler(scales, offsets))
     results = (
-        BasicOptimizer(
-            EnOptConfig.model_validate(enopt_config, context=transforms),
-            evaluator(),
-        )
-        .run()
+        BasicOptimizer(enopt_config, evaluator(), transforms=transforms)
+        .run(initial_values)
         .results
     )
     assert results is not None
 
     if offsets is not None:
-        initial_values = initial_values - offsets
         lower_bounds = lower_bounds - offsets
         upper_bounds = upper_bounds - offsets
     if scales is not None:
-        initial_values = initial_values / scales
         lower_bounds = lower_bounds / scales
         upper_bounds = upper_bounds / scales
     config = EnOptConfig.model_validate(enopt_config, context=transforms)
-    assert np.allclose(config.variables.initial_values, initial_values)
     assert np.allclose(config.variables.lower_bounds, lower_bounds)
     assert np.allclose(config.variables.upper_bounds, upper_bounds)
     result = results.evaluations.variables
@@ -480,9 +564,10 @@ def test_variables_scale_with_scaler(
 
 
 def test_variables_scale_linear_constraints_with_scaler(
-    enopt_config: Any,
-    evaluator: Any,
+    enopt_config: Any, evaluator: Any, external: str
 ) -> None:
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+
     coefficients = [[1, 0, 1], [0, 1, 1]]
     lower_bounds = [1.0, 0.75]
     upper_bounds = [1.0, 0.75]
@@ -515,37 +600,45 @@ def test_variables_scale_linear_constraints_with_scaler(
         (upper_bounds - offsets) / transformed_scales,
     )
 
-    results = BasicOptimizer(config, evaluator()).run().results
+    results = (
+        BasicOptimizer(enopt_config, evaluator(), transforms=transforms)
+        .run(initial_values)
+        .results
+    )
     assert results is not None
     assert results.evaluations.variables is not None
     assert np.allclose(results.evaluations.variables, [0.25, 0.0, 0.75], atol=0.02)
 
 
-def test_check_linear_constraints(enopt_config: Any, evaluator: Any) -> None:
+def test_check_linear_constraints(
+    enopt_config: Any, evaluator: Any, external: str
+) -> None:
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
     enopt_config["linear_constraints"] = {
         "coefficients": [[1, 1, 0], [1, 1, 0], [1, 1, 0]],
         "lower_bounds": [0.0, -np.inf, -1.0],
         "upper_bounds": [0.0, 1.0, np.inf],
     }
-    results1 = BasicOptimizer(enopt_config, evaluator()).run().results
+    results1 = BasicOptimizer(enopt_config, evaluator()).run(initial_values).results
     assert results1 is not None
 
     enopt_config["linear_constraints"]["lower_bounds"] = [0.0, -np.inf, -1.0]
     enopt_config["linear_constraints"]["upper_bounds"] = [0.0, 1.0, np.inf]
-    results2 = BasicOptimizer(enopt_config, evaluator()).run().results
+    results2 = BasicOptimizer(enopt_config, evaluator()).run(initial_values).results
     assert results2 is not None
     assert np.allclose(results1.evaluations.variables, results2.evaluations.variables)
 
     enopt_config["linear_constraints"]["lower_bounds"] = [1.0, -np.inf, 1.0]
     enopt_config["linear_constraints"]["upper_bounds"] = [1.0, -1.0, np.inf]
 
-    results = BasicOptimizer(enopt_config, evaluator()).run().results
+    results = BasicOptimizer(enopt_config, evaluator()).run(initial_values).results
     assert results is None
 
 
 def test_check_nonlinear_constraints(
-    enopt_config: Any, evaluator: Any, test_functions: Any
+    enopt_config: Any, evaluator: Any, test_functions: Any, external: str
 ) -> None:
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
     enopt_config["nonlinear_constraints"] = {
         "lower_bounds": [0.0, -np.inf, 0.0],
         "upper_bounds": [0.0, 0.0, np.inf],
@@ -558,27 +651,41 @@ def test_check_nonlinear_constraints(
         lambda variables, _: variables[0],
     )
 
-    results1 = BasicOptimizer(enopt_config, evaluator(test_functions)).run().results
+    results1 = (
+        BasicOptimizer(enopt_config, evaluator(test_functions))
+        .run(initial_values)
+        .results
+    )
     assert results1 is not None
 
     # Flipping the bounds should still work:
     enopt_config["nonlinear_constraints"]["lower_bounds"] = [0.0, -np.inf, 0.0]
     enopt_config["nonlinear_constraints"]["upper_bounds"] = [0.0, 0.0, np.inf]
-    results2 = BasicOptimizer(enopt_config, evaluator(test_functions)).run().results
+    results2 = (
+        BasicOptimizer(enopt_config, evaluator(test_functions))
+        .run(initial_values)
+        .results
+    )
     assert results2 is not None
     assert np.allclose(results1.evaluations.variables, results2.evaluations.variables)
 
     enopt_config["nonlinear_constraints"]["lower_bounds"] = [1.0, -np.inf, 1.0]
     enopt_config["nonlinear_constraints"]["upper_bounds"] = [1.0, -1.0, np.inf]
 
-    results = BasicOptimizer(enopt_config, evaluator(test_functions)).run().results
+    results = (
+        BasicOptimizer(enopt_config, evaluator(test_functions))
+        .run(initial_values)
+        .results
+    )
     assert results is None
 
 
-def test_optimizer_variables_subset(enopt_config: Any, evaluator: Any) -> None:
+def test_optimizer_variables_subset(
+    enopt_config: Any, evaluator: Any, external: str
+) -> None:
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
     # Set the second variable a constant value, this will not affect the
     # optimization of the other variables in this particular test problem:
-    enopt_config["variables"]["initial_values"] = [0.0, 1.0, 0.1]
     enopt_config["variables"]["mask"] = [True, False, True]
 
     def assert_gradient(results: tuple[Results, ...]) -> None:
@@ -591,7 +698,7 @@ def test_optimizer_variables_subset(enopt_config: Any, evaluator: Any) -> None:
     variables = (
         BasicOptimizer(enopt_config, evaluator())
         .set_results_callback(assert_gradient)
-        .run()
+        .run([0.0, 1.0, 0.1])
         .variables
     )
     assert variables is not None
@@ -599,13 +706,13 @@ def test_optimizer_variables_subset(enopt_config: Any, evaluator: Any) -> None:
 
 
 def test_optimizer_variables_subset_linear_constraints(
-    enopt_config: Any, evaluator: Any
+    enopt_config: Any, evaluator: Any, external: str
 ) -> None:
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
     # Set the second variable a constant value, this will not affect the
-    # optimization of the other variables in this particular test problem:
-    enopt_config["variables"]["initial_values"] = [0.0, 1.0, 0.1]
-    # The second and third constraints are dropped because they involve
-    # variables that are not optimized.
+    # optimization of the other variables in this particular test problem. The
+    # second and third constraints are dropped because they involve variables
+    # that are not optimized.
     enopt_config["linear_constraints"] = {
         "coefficients": [[1, 0, 1], [0, 1, 0], [1, 1, 1]],
         "lower_bounds": [1.0, 1.0, 2.0],
@@ -613,44 +720,93 @@ def test_optimizer_variables_subset_linear_constraints(
     }
     enopt_config["variables"]["mask"] = [True, False, True]
 
-    variables = BasicOptimizer(enopt_config, evaluator()).run().variables
+    variables = BasicOptimizer(enopt_config, evaluator()).run([0.0, 1.0, 0.1]).variables
     assert variables is not None
     assert np.allclose(variables, [0.25, 1.0, 0.75], atol=0.02)
 
 
-def test_parallelize(enopt_config: Any, evaluator: Any) -> None:
+def test_parallelize(enopt_config: Any, evaluator: Any, external: str) -> None:
     enopt_config["optimizer"] = {
-        "method": "differential_evolution",
+        "method": f"{external}{_DIFFERENTIAL_EVOLUTION}",
         "max_iterations": 15,
-        "options": {"seed": 123, "tol": 1e-10},
+        "options": {"rng": 123, "tol": 1e-10},
     }
     enopt_config["variables"]["lower_bounds"] = [0.15, 0.0, 0.0]
     enopt_config["variables"]["upper_bounds"] = [0.5, 0.5, 0.2]
-    enopt_config["variables"]["initial_values"][0] = 0.2
 
     enopt_config["optimizer"]["parallel"] = False
-    variables = BasicOptimizer(enopt_config, evaluator()).run().variables
+    variables = (
+        BasicOptimizer(enopt_config, evaluator())
+        .run([0.2, *initial_values[1:]])
+        .variables
+    )
     assert variables is not None
     assert np.allclose(variables, [0.15, 0.0, 0.2], atol=3e-2)
 
     enopt_config["optimizer"]["parallel"] = True
-    variables = BasicOptimizer(enopt_config, evaluator()).run().variables
+    variables = (
+        BasicOptimizer(enopt_config, evaluator())
+        .run([0.2, *initial_values[1:]])
+        .variables
+    )
     assert variables is not None
     assert np.allclose(variables, [0.15, 0.0, 0.2], atol=3e-2)
 
 
-def test_rng(enopt_config: Any, evaluator: Any) -> None:
-    variables1 = BasicOptimizer(enopt_config, evaluator()).run().variables
+def test_rng(enopt_config: Any, evaluator: Any, external: str) -> None:
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+    variables1 = BasicOptimizer(enopt_config, evaluator()).run(initial_values).variables
     assert variables1 is not None
     assert np.allclose(variables1, [0.0, 0.0, 0.5], atol=0.02)
 
-    variables2 = BasicOptimizer(enopt_config, evaluator()).run().variables
+    variables2 = BasicOptimizer(enopt_config, evaluator()).run(initial_values).variables
     assert variables2 is not None
     assert np.allclose(variables2, [0.0, 0.0, 0.5], atol=0.02)
     assert np.all(variables1 == variables2)
 
-    enopt_config["gradient"]["seed"] = (1, DEFAULT_SEED)
-    variables3 = BasicOptimizer(enopt_config, evaluator()).run().variables
+    enopt_config["variables"]["seed"] = (1, DEFAULT_SEED)
+    variables3 = BasicOptimizer(enopt_config, evaluator()).run(initial_values).variables
     assert variables3 is not None
     assert np.allclose(variables3, [0.0, 0.0, 0.5], atol=0.02)
     assert not np.all(variables3 == variables1)
+
+
+def test_arbitrary_objective_weights(
+    enopt_config: Any, evaluator: Any, external: str, test_functions: Any
+) -> None:
+    enopt_config["optimizer"]["method"] = f"{external}{_SLSQP}"
+    new_functions = (
+        *test_functions,
+        lambda variables, _: test_functions[1](variables, None),
+    )
+
+    enopt_config["objectives"]["weights"] = [0.75, 0.25, -0.25]
+    variables = (
+        BasicOptimizer(enopt_config, evaluator(new_functions))
+        .run(initial_values)
+        .variables
+    )
+    assert variables is not None
+    assert not np.allclose(variables, [0, 0, 0.5], atol=0.02)
+
+    enopt_config["objectives"]["weights"] = [0.75, 0.25, 0.0]
+    variables = (
+        BasicOptimizer(enopt_config, evaluator(new_functions))
+        .run(initial_values)
+        .variables
+    )
+    assert variables is not None
+    assert np.allclose(variables, [0, 0, 0.5], atol=0.02)
+
+    enopt_config["objectives"]["weights"] = [0.75, 0.5, -0.25]
+    variables = (
+        BasicOptimizer(enopt_config, evaluator(new_functions))
+        .run(initial_values)
+        .variables
+    )
+    assert variables is not None
+    assert np.allclose(variables, [0, 0, 0.5], atol=0.02)
+
+    enopt_config["objectives"]["weights"] = [-0.75, -0.25]
+    with pytest.raises(ValidationError, match="The sum of weights is not positive"):
+        EnOptConfig.model_validate(enopt_config)
