@@ -12,6 +12,8 @@ from ropt.plugins.realization_filter.default import (
 )
 from ropt.results import FunctionResults, GradientResults, Results
 
+initial_values = 3 * [0]
+
 
 @pytest.fixture(name="enopt_config")
 def enopt_config_fixture() -> dict[str, Any]:
@@ -24,14 +26,14 @@ def enopt_config_fixture() -> dict[str, Any]:
         },
         "gradient": {
             "number_of_perturbations": 5,
-            "perturbation_magnitudes": 0.01,
             "evaluation_policy": "speculative",
         },
         "realizations": {
             "weights": 5 * [1.0],
         },
         "variables": {
-            "initial_values": 3 * [0],
+            "variable_count": 3,
+            "perturbation_magnitudes": 0.01,
         },
     }
 
@@ -81,25 +83,25 @@ def test__get_cvar_weights_from_percentile(
 
 def _objective_function(
     variables: NDArray[np.float64],
-    context: Any,
+    realization: int,
     target: NDArray[np.float64],
 ) -> float:
     diff: NDArray[np.float64] = variables - target
     # Make sure that some realizations yield a different result:
-    if context.realization % 2 == 0:
+    if realization % 2 == 0:
         diff += 1.0
     result = np.sum(diff**2)
     # Make sure the other realizations have a worse result:
-    if context.realization % 2 == 1:
+    if realization % 2 == 1:
         result += 10.0
     return float(result)
 
 
-def _constraint_function(variables: NDArray[np.float64], context: Any) -> float:
+def _constraint_function(variables: NDArray[np.float64], realization: int) -> float:
     # Track how often this function is called.
     result = variables[0] + variables[2]
     # Break some realizations, same as in the distance function:
-    if context.realization % 2 == 0:
+    if realization % 2 == 0:
         result = variables[0] + variables[2] - 10.0
     return float(result)
 
@@ -121,7 +123,9 @@ def test_sort_filter_on_objectives(
 
     enopt_config["gradient"]["evaluation_policy"] = evaluation_policy
 
-    variables = BasicOptimizer(enopt_config, evaluator(functions)).run().variables
+    variables = (
+        BasicOptimizer(enopt_config, evaluator(functions)).run(initial_values).variables
+    )
     assert variables is not None
     assert not np.allclose(variables, [0.0, 0.0, 0.5], atol=0.02)
 
@@ -139,12 +143,9 @@ def test_sort_filter_on_objectives(
 
     result_list: list[Results] = []
     results = (
-        BasicOptimizer(
-            enopt_config,
-            evaluator(functions),
-        )
+        BasicOptimizer(enopt_config, evaluator(functions))
         .set_results_callback(partial(_track_results, result_list=result_list))
-        .run()
+        .run(initial_values)
         .results
     )
     assert results is not None
@@ -198,12 +199,9 @@ def test_sort_filter_on_objectives_with_constraints(
     enopt_config["nonlinear_constraints"]["realization_filters"] = [0]
     result_list: list[Results] = []
     results = (
-        BasicOptimizer(
-            enopt_config,
-            evaluator(functions),
-        )
+        BasicOptimizer(enopt_config, evaluator(functions))
         .set_results_callback(partial(_track_results, result_list=result_list))
-        .run()
+        .run(initial_values)
         .results
     )
     assert results is not None
@@ -263,14 +261,11 @@ def test_sort_filter_on_constraints(
     enopt_config["nonlinear_constraints"]["realization_filters"] = [0]
     result_list: list[Results] = []
     results = (
-        BasicOptimizer(
-            enopt_config,
-            evaluator(functions),
-        )
+        BasicOptimizer(enopt_config, evaluator(functions))
         .set_results_callback(
             partial(_track_results, result_list=result_list),
         )
-        .run()
+        .run(initial_values)
         .results
     )
     assert results is not None
@@ -342,7 +337,7 @@ def test_sort_filter_mixed(  # noqa: C901
     results = (
         BasicOptimizer(enopt_config, evaluator(functions))
         .set_results_callback(_add_objective)
-        .run()
+        .run(initial_values)
         .results
     )
     assert results is not None
@@ -380,7 +375,7 @@ def test_sort_filter_mixed(  # noqa: C901
     results = (
         BasicOptimizer(enopt_config, evaluator(functions))
         .set_results_callback(_add_objective)
-        .run()
+        .run(initial_values)
         .results
     )
     assert results is not None
@@ -391,18 +386,6 @@ def test_sort_filter_mixed(  # noqa: C901
             assert result.realizations is not None
             assert result.realizations.objective_weights is not None
             filtered = result.realizations.objective_weights[0, :] == 0.0
-        if isinstance(result, GradientResults):
-            if evaluation_policy == "separate":
-                assert np.all(
-                    result.evaluations.perturbed_objectives[filtered, :, :2] == 0.0
-                )
-            else:
-                assert np.all(
-                    result.evaluations.perturbed_objectives[filtered, :, :2] != 0.0
-                )
-            assert np.all(
-                result.evaluations.perturbed_objectives[filtered, :, 2:] != 0.0
-            )
 
     # The first objective values should differ.
     assert objective_values[0] != objective_values[1]
@@ -421,7 +404,9 @@ def test_cvar_filter_on_objectives(
 
     enopt_config["gradient"]["evaluation_policy"] = evaluation_policy
 
-    variables = BasicOptimizer(enopt_config, evaluator(functions)).run().variables
+    variables = (
+        BasicOptimizer(enopt_config, evaluator(functions)).run(initial_values).variables
+    )
     assert variables is not None
     assert not np.allclose(variables, [0.0, 0.0, 0.5], atol=0.02)
 
@@ -437,14 +422,11 @@ def test_cvar_filter_on_objectives(
     enopt_config["objectives"]["realization_filters"] = [0, 0]
     result_list: list[Results] = []
     results = (
-        BasicOptimizer(
-            enopt_config,
-            evaluator(functions),
-        )
+        BasicOptimizer(enopt_config, evaluator(functions))
         .set_results_callback(
             partial(_track_results, result_list=result_list),
         )
-        .run()
+        .run(initial_values)
         .results
     )
     assert results is not None
@@ -497,14 +479,11 @@ def test_cvar_filter_on_objectives_with_constraints(
     enopt_config["nonlinear_constraints"]["realization_filters"] = [0]
     result_list: list[Results] = []
     results = (
-        BasicOptimizer(
-            enopt_config,
-            evaluator(functions),
-        )
+        BasicOptimizer(enopt_config, evaluator(functions))
         .set_results_callback(
             partial(_track_results, result_list=result_list),
         )
-        .run()
+        .run(initial_values)
         .results
     )
     assert results is not None
@@ -564,14 +543,11 @@ def test_cvar_filter_on_constraints(
     enopt_config["nonlinear_constraints"]["realization_filters"] = [0]
     result_list: list[Results] = []
     results = (
-        BasicOptimizer(
-            enopt_config,
-            evaluator(functions),
-        )
+        BasicOptimizer(enopt_config, evaluator(functions))
         .set_results_callback(
             partial(_track_results, result_list=result_list),
         )
-        .run()
+        .run(initial_values)
         .results
     )
     assert results is not None
@@ -601,7 +577,7 @@ def test_cvar_filter_on_constraints(
 
 
 @pytest.mark.parametrize("evaluation_policy", ["separate", "speculative"])
-def test_cvar_filter_mixed(  # noqa: C901
+def test_cvar_filter_mixed(
     enopt_config: Any,
     evaluator: Any,
     evaluation_policy: Literal["speculative", "separate", "auto"],
@@ -641,7 +617,7 @@ def test_cvar_filter_mixed(  # noqa: C901
     results = (
         BasicOptimizer(enopt_config, evaluator(functions))
         .set_results_callback(_add_objective)
-        .run()
+        .run(initial_values)
         .results
     )
     assert results is not None
@@ -678,7 +654,7 @@ def test_cvar_filter_mixed(  # noqa: C901
     results = (
         BasicOptimizer(enopt_config, evaluator(functions))
         .set_results_callback(_add_objective)
-        .run()
+        .run(initial_values)
         .results
     )
     assert results is not None
@@ -689,18 +665,6 @@ def test_cvar_filter_mixed(  # noqa: C901
             assert result.realizations is not None
             assert result.realizations.objective_weights is not None
             filtered = result.realizations.objective_weights[0, :] == 0.0
-        if isinstance(result, GradientResults):
-            if evaluation_policy == "separate":
-                assert np.all(
-                    result.evaluations.perturbed_objectives[filtered, :, :2] == 0.0
-                )
-            else:
-                assert np.all(
-                    result.evaluations.perturbed_objectives[filtered, :, :2] != 0.0
-                )
-            assert np.all(
-                result.evaluations.perturbed_objectives[filtered, :, 2:] != 0.0
-            )
 
     # The first objective values should differ.
     assert objective_values[0] != objective_values[1]
